@@ -4,25 +4,25 @@
  * Authors:  Jacob Berkman
  */
 
-#ifndef __FLOATING_WIDGET_H__
-#define __FLOATING_WIDGET_H__
+#ifndef FLOATING_WIDGET_H
+#define FLOATING_WIDGET_H
 
 #include "basep-widget.h"
 
 BEGIN_GNOME_DECLS
 
-#define FLOATING_POS_TYPE            (floating_pos_get_type ())
-#define FLOATING_POS(o)              (GTK_CHECK_CAST ((o), FLOATING_POS_TYPE, FloatingPos))
-#define FLOATING_POS_CLASS(k)        (GTK_CHECK_CLASS_CAST ((k), FLOATING_POS_TYPE, FloatingPosClass))
-#define IS_FLOATING_POS(o)          (GTK_CHECK_TYPE ((o), FLOATING_POS_TYPE))
-#define IS_FLOATING_POS_CLASS(k)    (GTK_CHECK_CLASS_TYPE ((k), FLOATING_POS_TYPE))
+#define TYPE_FLOATING_POS            (floating_pos_get_type ())
+#define FLOATING_POS(o)              (GTK_CHECK_CAST ((o), TYPE_FLOATING_POS, FloatingPos))
+#define FLOATING_POS_CLASS(k)        (GTK_CHECK_CLASS_CAST ((k), TYPE_FLOATING_POS, FloatingPosClass))
+#define IS_FLOATING_POS(o)           (GTK_CHECK_TYPE ((o), TYPE_FLOATING_POS))
+#define IS_FLOATING_POS_CLASS(k)     (GTK_CHECK_CLASS_TYPE ((k), TYPE_FLOATING_POS))
 
-#define FLOATING_WIDGET_TYPE         (BASEP_WIDGET_TYPE)
+#define TYPE_FLOATING_WIDGET         (TYPE_BASEP_WIDGET)
 #define FLOATING_WIDGET(o)           (BASEP_WIDGET(o))
 #define FLOATING_WIDGET_CLASS(k)     (BASEP_WIDGET_CLASS(k))
 #define IS_FLOATING_WIDGET(o)        (IS_BASEP_WIDGET(o) && IS_FLOATING_POS(BASEP_WIDGET(o)->pos))
 /* this is not reliable */
-#define IS_FLOATING_WIDGET_CLASS(k) (IS_BASEP_WIDGET_CLASS(k))
+#define IS_FLOATING_WIDGET_CLASS(k)  (IS_BASEP_WIDGET_CLASS(k))
 
 typedef BasePWidget            FloatingWidget;
 typedef BasePWidgetClass       FloatingWidgetClass;
@@ -44,7 +44,7 @@ struct _FloatingPosClass {
 			       gint x, gint y);
 };
 
-GtkType floating_pos_get_type (void);
+GtkType floating_pos_get_type (void) G_GNUC_CONST;
 GtkWidget *floating_widget_new (gint16 x, gint16 y,
 				PanelOrientType orient,
 				BasePMode mode,
