@@ -48,10 +48,10 @@
 #include <libart_lgpl/art_alphagamma.h>
 
 /* panel-util makes references to this, so yes this is UGLY UGLY UGLY to make
- * it available, but we're not using those parts, so we just want the compiler
- * to shut up */
-GlobalConfig global_config;
-static GlobalConfig loaded_config;
+ * it available (and not static), but we're not using those parts, so we just
+ * want the compiler to shut up */
+GlobalConfig global_config = {0};
+static GlobalConfig loaded_config = {0};
 
 /* animation page */
 static GtkWidget *enable_animations_cb;
@@ -77,6 +77,7 @@ static GtkWidget *entry_down[LAST_TILE];
 static GtkWidget *hide_panel_frame_cb;
 static GtkWidget *tile_when_over_cb;
 static GtkWidget *saturate_when_over_cb;
+static GtkWidget *fast_button_scaling_cb;
 
 
 /* applet page*/
@@ -92,6 +93,7 @@ static GtkWidget *show_dot_buttons_cb;
 static GtkWidget *off_panel_popups_cb;
 static GtkWidget *hungry_menus_cb;
 static GtkWidget *use_large_icons_cb;
+static GtkWidget *merge_menus_cb;
 
 typedef struct {
 	int inline_flag;
@@ -106,14 +108,13 @@ typedef struct {
 /* don't forget to update the table size when changing the num of elements
    in this array */
 static MenuOptions menu_options[] = {
-	{ MAIN_MENU_SYSTEM,  MAIN_MENU_SYSTEM_SUB,  N_("Programs: ") },
-	{ MAIN_MENU_USER,    MAIN_MENU_USER_SUB,    N_("Favorites: ") },
-	{ MAIN_MENU_APPLETS, MAIN_MENU_APPLETS_SUB, N_("Applets: ") },
-	{ MAIN_MENU_REDHAT,  MAIN_MENU_REDHAT_SUB,  N_("AnotherLevel: ") },
-	{ MAIN_MENU_KDE,     MAIN_MENU_KDE_SUB,     N_("KDE: ") },
-	{ MAIN_MENU_DEBIAN,  MAIN_MENU_DEBIAN_SUB,  N_("Debian: ") },
-	{ MAIN_MENU_PANEL,   MAIN_MENU_PANEL_SUB,   N_("Panel: ") },
-	{ MAIN_MENU_DESKTOP, MAIN_MENU_DESKTOP_SUB, N_("Desktop: ") },
+	{ MAIN_MENU_SYSTEM,       MAIN_MENU_SYSTEM_SUB,        N_("Programs: ") },
+	{ MAIN_MENU_USER,         MAIN_MENU_USER_SUB,          N_("Favorites: ") },
+	{ MAIN_MENU_APPLETS,      MAIN_MENU_APPLETS_SUB,       N_("Applets: ") },
+	{ MAIN_MENU_DISTRIBUTION, MAIN_MENU_DISTRIBUTION_SUB,  N_("Distribution: ") },
+	{ MAIN_MENU_KDE,          MAIN_MENU_KDE_SUB,           N_("KDE: ") },
+	{ MAIN_MENU_PANEL,        MAIN_MENU_PANEL_SUB,         N_("Panel: ") },
+	{ MAIN_MENU_DESKTOP,      MAIN_MENU_DESKTOP_SUB,       N_("Desktop: ") },
 	{ 0 }
 };
 
@@ -166,6 +167,32 @@ void transform_pixbuf(guchar *dst, int x0, int y0, int x1, int y1, int drs,
 
 #include "nothing.cP"
 
+static void
+set_config (GlobalConfig *dest, GlobalConfig *src)
+{
+	int i;
+
+	g_return_if_fail (dest != NULL);
+	g_return_if_fail (src != NULL);
+
+	for (i = 0; i < LAST_TILE; i++) {
+		g_free (dest->tile_up[i]);
+		g_free (dest->tile_down[i]);
+	}
+	g_free (dest->menu_key);
+	g_free (dest->run_key);
+
+	*dest = *src;
+
+	for (i = 0; i < LAST_TILE; i++) {
+		dest->tile_up[i] = g_strdup (dest->tile_up[i]);
+		dest->tile_down[i] = g_strdup (dest->tile_down[i]);
+	}
+	dest->menu_key = g_strdup (dest->menu_key);
+	dest->run_key = g_strdup (dest->run_key);
+}
+
+
 static void 
 changed_cb(void)
 {
@@ -337,9 +364,11 @@ animation_notebook_page(void)
 }
 
 static char *
-get_full_tile(char *file)
+get_full_tile(const char *file)
 {
-	if(g_path_is_absolute(file))
+	if (file == NULL)
+		return NULL;
+	else if (g_path_is_absolute(file))
 		return g_strdup(file);
 	else
 		return gnome_unconditional_pixmap_file(file);
@@ -355,6 +384,8 @@ sync_buttons_page_with_config(GlobalConfig *conf)
 				    conf->tile_when_over);
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(saturate_when_over_cb),
 				    conf->saturate_when_over);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(fast_button_scaling_cb),
+				    conf->fast_button_scaling);
 	
 	for(i=0;i<LAST_TILE;i++) {
 		char *file;
@@ -381,6 +412,8 @@ sync_config_with_buttons_page(GlobalConfig *conf)
 		GTK_TOGGLE_BUTTON(tile_when_over_cb)->active;
 	conf->saturate_when_over =
 		GTK_TOGGLE_BUTTON(saturate_when_over_cb)->active;
+	conf->fast_button_scaling =
+		GTK_TOGGLE_BUTTON(fast_button_scaling_cb)->active;
 
 	for(i=0;i<LAST_TILE;i++) {
 		conf->tiles_enabled[i] =
@@ -581,6 +614,12 @@ buttons_notebook_page (void)
 	  gtk_signal_connect (GTK_OBJECT (saturate_when_over_cb), "toggled",
 			      GTK_SIGNAL_FUNC (changed_cb), NULL);
 	  gtk_box_pack_start (GTK_BOX (vbox), saturate_when_over_cb, FALSE, FALSE, 0);
+
+	  /* Fast but low quality scaling (Nearest versus Hyperbolic) */
+	  fast_button_scaling_cb = gtk_check_button_new_with_label (_("Fast but low quality scaling of button icons"));
+	  gtk_signal_connect (GTK_OBJECT (fast_button_scaling_cb), "toggled",
+			      GTK_SIGNAL_FUNC (changed_cb), NULL);
+	  gtk_box_pack_start (GTK_BOX (vbox), fast_button_scaling_cb, FALSE, FALSE, 0);
 	  
 	  return (vbox);
 }
@@ -685,6 +724,8 @@ sync_menu_page_with_config(GlobalConfig *conf)
 				    conf->hungry_menus);
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(use_large_icons_cb),
 				    conf->use_large_icons);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(merge_menus_cb),
+				    conf->merge_menus);
 
 	for (opt = menu_options; opt->inline_flag; ++opt) {
 		if (conf->menu_flags & opt->inline_flag)
@@ -709,6 +750,8 @@ sync_config_with_menu_page(GlobalConfig *conf)
 		GTK_TOGGLE_BUTTON(hungry_menus_cb)->active;
 	conf->use_large_icons =
 		GTK_TOGGLE_BUTTON(use_large_icons_cb)->active;
+	conf->merge_menus =
+		GTK_TOGGLE_BUTTON(merge_menus_cb)->active;
 	conf->menu_flags = 0;
 	for (opt = menu_options; opt->inline_flag; ++opt) {
 		if (GTK_TOGGLE_BUTTON (opt->inline_rb)->active)
@@ -764,9 +807,15 @@ menu_notebook_page(void)
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
 	
 	/* table for frame */
-	table = gtk_table_new(2,2,FALSE);
+	table = gtk_table_new(3, 2, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER (table), GNOME_PAD_SMALL);
 	gtk_container_add (GTK_CONTAINER (frame), table);
+
+	/* large icons */
+	use_large_icons_cb = gtk_check_button_new_with_label (_("Use large icons"));
+	gtk_signal_connect (GTK_OBJECT (use_large_icons_cb), "toggled",
+			    GTK_SIGNAL_FUNC (changed_cb), NULL);
+	gtk_table_attach_defaults(GTK_TABLE(table),use_large_icons_cb, 0,1,0,1);
 	
 	/* Dot Buttons */
 	show_dot_buttons_cb = gtk_check_button_new_with_label (_("Show [...] buttons"));
@@ -786,11 +835,12 @@ menu_notebook_page(void)
 			    GTK_SIGNAL_FUNC (changed_cb), NULL);
 	gtk_table_attach_defaults(GTK_TABLE(table),hungry_menus_cb, 1,2,1,2);
 
-	/* large icons */
-	use_large_icons_cb = gtk_check_button_new_with_label (_("Use large icons"));
-	gtk_signal_connect (GTK_OBJECT (use_large_icons_cb), "toggled",
-			    GTK_SIGNAL_FUNC (changed_cb), NULL);
-	gtk_table_attach_defaults(GTK_TABLE(table),use_large_icons_cb, 0,1,0,1);
+	/* Merge system menus */
+	merge_menus_cb = gtk_check_button_new_with_label (_("Merge in system menus"));
+	gtk_signal_connect (GTK_OBJECT (merge_menus_cb), "toggled", 
+			    GTK_SIGNAL_FUNC (changed_cb),  NULL);
+	gtk_table_attach_defaults(GTK_TABLE(table), merge_menus_cb,
+				  0, 1, 2, 3);
 
 	/* Menu frame */
 	frame = gtk_frame_new (_("Global menu"));
@@ -824,9 +874,9 @@ sync_misc_page_with_config(GlobalConfig *conf)
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(keys_enabled_cb),
 				    conf->keys_enabled);
 	gtk_entry_set_text (GTK_ENTRY(menu_key_entry),
-			    conf->menu_key?conf->menu_key:"");
+			    conf->menu_key ? conf->menu_key : "");
 	gtk_entry_set_text (GTK_ENTRY(run_key_entry),
-			    conf->run_key?conf->run_key:"");
+			    conf->run_key ? conf->run_key : "");
 }
 static void
 sync_config_with_misc_page(GlobalConfig *conf)
@@ -1079,6 +1129,9 @@ loadup_vals(void)
 	global_config.use_large_icons =
 		gnome_config_get_bool("use_large_icons=TRUE");
 
+	global_config.merge_menus =
+		gnome_config_get_bool("merge_menus=TRUE");
+
 	global_config.off_panel_popups =
 		gnome_config_get_bool("off_panel_popups=TRUE");
 		
@@ -1108,9 +1161,8 @@ loadup_vals(void)
 
 	g_string_sprintf(buf,"menu_flags=%d", 
 			 (int)(MAIN_MENU_SYSTEM_SUB | MAIN_MENU_USER_SUB|
-			       MAIN_MENU_APPLETS_SUB | MAIN_MENU_REDHAT_SUB|
-			       MAIN_MENU_DEBIAN_SUB | MAIN_MENU_KDE_SUB|
-			       MAIN_MENU_PANEL | MAIN_MENU_DESKTOP));
+			       MAIN_MENU_APPLETS_SUB | MAIN_MENU_DISTRIBUTION_SUB|
+			       MAIN_MENU_KDE_SUB | MAIN_MENU_PANEL | MAIN_MENU_DESKTOP));
 	global_config.menu_flags = gnome_config_get_int(buf->str);
 
 	global_config.keys_enabled = gnome_config_get_bool("keys_enabled=TRUE");
@@ -1137,6 +1189,7 @@ loadup_vals(void)
 	global_config.tile_when_over = gnome_config_get_bool("tile_when_over=FALSE");
 	global_config.saturate_when_over = gnome_config_get_bool("saturate_when_over=TRUE");
 	global_config.confirm_panel_remove = gnome_config_get_bool("confirm_panel_remove=TRUE");
+	global_config.fast_button_scaling = gnome_config_get_bool("fast_button_scaling=FALSE");
 
 	for(i=0;i<LAST_TILE;i++) {
 		g_string_sprintf(buf,"new_tiles_enabled_%d=FALSE",i);
@@ -1208,6 +1261,8 @@ write_config(GlobalConfig *conf)
 			      conf->hungry_menus);
 	gnome_config_set_bool("use_large_icons",
 			      conf->use_large_icons);
+	gnome_config_set_bool("merge_menus",
+			      conf->merge_menus);
 	gnome_config_set_bool("off_panel_popups",
 			      conf->off_panel_popups);
 	gnome_config_set_bool("disable_animations",
@@ -1234,6 +1289,7 @@ write_config(GlobalConfig *conf)
 	gnome_config_set_bool("keys_enabled", conf->keys_enabled);
 	gnome_config_set_string("menu_key", conf->menu_key);
 	gnome_config_set_string("run_key", conf->run_key);
+	gnome_config_set_bool("fast_button_scaling", conf->fast_button_scaling);
 			     
 	buf = g_string_new(NULL);
 	for(i=0;i<LAST_TILE;i++) {
@@ -1280,8 +1336,11 @@ revert(GtkWidget *capplet, gpointer data)
 	sync_applets_page_with_config(&loaded_config);
 	sync_menu_page_with_config(&loaded_config);
 	sync_misc_page_with_config(&loaded_config);
-	global_config = loaded_config;
-	write_config(&global_config);
+
+	set_config (&global_config, &loaded_config);
+
+	write_config(&loaded_config);
+
 	changing = FALSE;
 }
 
@@ -1361,7 +1420,7 @@ main (int argc, char **argv)
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-panel.png");
 	loadup_vals();
 	
-	loaded_config = global_config;
+	set_config (&loaded_config, &global_config);
 
 	capplet = capplet_widget_new();
 
