@@ -41,6 +41,8 @@ struct _GlobalConfig {
 	  guint run_state;    /* from the above */
 	int menu_flags;
 	gboolean use_large_icons;
+	gboolean merge_menus;
+	gboolean fast_button_scaling;
 };
 
 #endif /* PANEL_CONFIG_GLOBAL_H */
